@@ -1,3 +1,4 @@
+#!/usr/bin/python
 #
 # Copyright (C) 2006 Google Inc.
 #
@@ -113,25 +114,29 @@ class GDataService(atom.service.AtomService):
 
   def __init__(self, email=None, password=None, account_type='HOSTED_OR_GOOGLE',
                service=None, source=None, server=None, 
-               additional_headers=None):
+               additional_headers=None, handler=None):
     """Creates an object of type GDataService.
 
     Args:
       email: string (optional) The user's email address, used for
-             authentication.
+          authentication.
       password: string (optional) The user's password.
       account_type: string (optional) The type of account to use. Use
-              'GOOGLE' for regular Google accounts or 'HOSTED' for Google
-              Apps accounts, or 'HOSTED_OR_GOOGLE' to try finding a HOSTED
-              account first and, if it doesn't exist, try finding a regular
-              GOOGLE account. Default value: 'HOSTED_OR_GOOGLE'.
+          'GOOGLE' for regular Google accounts or 'HOSTED' for Google
+          Apps accounts, or 'HOSTED_OR_GOOGLE' to try finding a HOSTED
+          account first and, if it doesn't exist, try finding a regular
+          GOOGLE account. Default value: 'HOSTED_OR_GOOGLE'.
       service: string (optional) The desired service for which credentials
-               will be obtained.
+          will be obtained.
       source: string (optional) The name of the user's application.
       server: string (optional) The name of the server to which a connection
-              will be opened. Default value: 'base.google.com'.
+          will be opened. Default value: 'base.google.com'.
       additional_headers: dictionary (optional) Any additional headers which 
-                          should be included with CRUD operations.
+          should be included with CRUD operations.
+      handler: class (optional) The class whose Get, Post, Put, Delete, 
+          CreateConnection and PrepareConnection methods should be used 
+          when making requests to the server. The default value is 
+          atom.service.AtomService.
     """
 
     self.email = email
@@ -140,6 +145,7 @@ class GDataService(atom.service.AtomService):
     self.service = service
     self.server = server
     self.additional_headers = additional_headers or {}
+    self.handler = handler or atom.service.AtomService
     self.__SetSource(source)
     self.__auth_token = None
     self.__captcha_token = None
@@ -242,7 +248,7 @@ class GDataService(atom.service.AtomService):
   def __SetSource(self, new_source):
     self.__source = new_source
     # Update the UserAgent header to include the new application name.
-    self.additional_headers['User-Agent'] = '%s GData-Python/1.0.9' % self.__source
+    self.additional_headers['User-Agent'] = '%s GData-Python/1.0.10' % self.__source
 
   source = property(__GetSource, __SetSource, 
       doc="""The source is the name of the application making the request. 
@@ -277,7 +283,7 @@ class GDataService(atom.service.AtomService):
         captcha_token, captcha_response)
 
     # Open a connection to the authentication server.
-    (auth_connection, uri) = self._PrepareConnection(AUTH_SERVER_HOST)
+    (auth_connection, uri) = self.PrepareConnection(AUTH_SERVER_HOST)
 
     # Begin the POST request to the client login service.
     auth_connection.putrequest('POST', '/accounts/ClientLogin')
@@ -388,7 +394,7 @@ class GDataService(atom.service.AtomService):
     if not self.__auth_token.startswith(AUTHSUB_AUTH_LABEL):
       raise NonAuthSubToken
 
-    (upgrade_connection, uri) = self._PrepareConnection(
+    (upgrade_connection, uri) = self.PrepareConnection(
         AUTH_SERVER_HOST)
     upgrade_connection.putrequest('GET', '/accounts/AuthSubSessionToken')
     
@@ -403,7 +409,7 @@ class GDataService(atom.service.AtomService):
     if response.status == 200:
       for response_line in response_body.splitlines():
         if response_line.startswith('Token='):
-          self.__auth_token = response_line.lstrip('Token=')
+          self.SetAuthSubToken(response_line.lstrip('Token='))
 
   def RevokeAuthSubToken(self):
     """Revokes an existing AuthSub token.
@@ -415,7 +421,7 @@ class GDataService(atom.service.AtomService):
     if not self.__auth_token.startswith(AUTHSUB_AUTH_LABEL):
       raise NonAuthSubToken
     
-    (revoke_connection, uri) = self._PrepareConnection(
+    (revoke_connection, uri) = self.PrepareConnection(
         AUTH_SERVER_HOST)
     revoke_connection.putrequest('GET', '/accounts/AuthSubRevokeToken')
     
@@ -428,6 +434,9 @@ class GDataService(atom.service.AtomService):
     if response.status == 200:
       self.__auth_token = None
 
+  def PrepareConnection(self, full_uri):
+    return self.handler.PrepareConnection(self, full_uri)
+
   # CRUD operations
   def Get(self, uri, extra_headers=None, redirects_remaining=4, encoding='UTF-8', converter=None):
     """Query the GData API with the given URI
@@ -481,7 +490,7 @@ class GDataService(atom.service.AtomService):
         else:
           uri += '?gsessionid=%s' % (self.__gsessionid,)
 
-    server_response = atom.service.AtomService.Get(self, uri, extra_headers)
+    server_response = self.handler.Get(self, uri, extra_headers)
     result_body = server_response.read()
 
     if server_response.status == 200:
@@ -525,7 +534,7 @@ class GDataService(atom.service.AtomService):
     """Returns a MediaSource containing media and its metadata from the given
     URI string.
     """
-    connection = atom.service.AtomService._CreateConnection(self, uri, 'GET',
+    connection = self.handler.CreateConnection(self, uri, 'GET', 
         extra_headers)
     response_handle = connection.getresponse()
     return gdata.MediaSource(response_handle, response_handle.getheader('Content-Type'),
@@ -675,7 +684,7 @@ class GDataService(atom.service.AtomService):
           len(multipart[1]) + len(multipart[2]) +
           len(data_str) + media_source.content_length)
           
-      insert_connection = atom.service.AtomService._CreateConnection(self,
+      insert_connection = self.handler.CreateConnection(self, 
           uri, 'POST', extra_headers, url_params, escape_params)
 
       insert_connection.send(multipart[0])
@@ -695,7 +704,7 @@ class GDataService(atom.service.AtomService):
     elif media_source:
       extra_headers['Content-Type'] = media_source.content_type
       extra_headers['Content-Length'] = media_source.content_length
-      insert_connection = atom.service.AtomService._CreateConnection(self, uri,
+      insert_connection = self.handler.CreateConnection(self, uri,
           'POST', extra_headers, url_params, escape_params)
 
       while 1:
@@ -710,7 +719,7 @@ class GDataService(atom.service.AtomService):
       http_data = data
       content_type = 'application/atom+xml'
 
-      server_response = atom.service.AtomService.Post(self, http_data, uri,
+      server_response = self.handler.Post(self, http_data, uri,
           extra_headers, url_params, escape_params, content_type)
       result_body = server_response.read()
 
@@ -814,7 +823,7 @@ class GDataService(atom.service.AtomService):
           len(multipart[1]) + len(multipart[2]) +
           len(data_str) + media_source.content_length)
           
-      insert_connection = atom.service.AtomService._CreateConnection(self, uri,
+      insert_connection = self.handler.CreateConnection(self, uri,
           'PUT', extra_headers, url_params, escape_params)
 
       insert_connection.send(multipart[0])
@@ -834,7 +843,7 @@ class GDataService(atom.service.AtomService):
     elif media_source:
       extra_headers['Content-Type'] = media_source.content_type
       extra_headers['Content-Length'] = media_source.content_length
-      insert_connection = atom.service.AtomService._CreateConnection(self, uri,
+      insert_connection = self.handler.CreateConnection(self, uri,
           'PUT', extra_headers, url_params, escape_params)
 
       while 1:
@@ -848,7 +857,7 @@ class GDataService(atom.service.AtomService):
       http_data = data
       content_type = 'application/atom+xml'
 
-      server_response = atom.service.AtomService.Put(self, http_data, uri,
+      server_response = self.handler.Put(self, http_data, uri,
           extra_headers, url_params, escape_params, content_type)
       result_body = server_response.read()
 
@@ -921,7 +930,7 @@ class GDataService(atom.service.AtomService):
         else:
           uri += '?gsessionid=%s' % (self.__gsessionid,)
                                                   
-    server_response = atom.service.AtomService.Delete(self, uri,
+    server_response = self.handler.Delete(self, uri,
         extra_headers, url_params, escape_params)
     result_body = server_response.read()
 
@@ -1107,6 +1116,18 @@ class Query(dict):
   max_results = property(_GetMaxResults, _SetMaxResults,
       doc="""The feed query's max-results parameter""")
 
+  def _GetOrderBy(self):
+    if 'orderby' in self.keys():
+      return self['orderby']
+    else:
+      return None
+ 
+  def _SetOrderBy(self, query):
+    self['orderby'] = query
+  
+  orderby = property(_GetOrderBy, _SetOrderBy, 
+      doc="""The feed query's orderby parameter""")
+
 
   def ToUri(self):
     q_feed = self.feed or ''
